#define DISCOUNT 0.99999 // Discount factor

/* Resolution parameters */
#define N_NEIGHBORS 4
#define N_POSITIONS_0 51
#define N_VELOCITIES_0 51
#define N_TOTAL_STATES (N_POSITIONS_0*N_VELOCITIES_0)

/* What we put in policy if no answer found */
#define BAD_U 0

#define WEIGHTED_AVE_SCALE -7.0   // used in weighted averaging of values
/* -7 ok */

#define LQR_BORDER 1e-4   // fuzz to add to domain of LQR terminal controller 
#define LQR_PARAM 0.3     // how centered desired state must be in goal cell
#define LQR_VALUE_THRESHOLD 0.021 // A cutoff for use of the LQR controller.

/* Used to manage bad entries in tables */
#define INFINITE_COST 1e30
#define COST_CUTOFF 1e9

/* Resolutions go here. This should be treated as read-only */
static int n_states[N_STATE_DIMENSIONS] = { N_POSITIONS_0, N_VELOCITIES_0 };

/* Bounds on what we pay attention to. These should be treated as read-only */
static float min_states[N_STATE_DIMENSIONS] = { -3.0, -7.0 };
static float max_states[N_STATE_DIMENSIONS] = { 4.5, 9.0 };
static float min_actions[N_ACTION_DIMENSIONS] = { -1.0 };
static float max_actions[N_ACTION_DIMENSIONS] = { 1.0 };

/* LQR stuff. These should be treated as read-only */
/* For some reason this does not work.
static float lqr_k[N_ACTION_DIMENSIONS][N_STATE_DIMENSIONS] =
...
static float lqr_s[N_STATE_DIMENSIONS][N_STATE_DIMENSIONS] =
...
*/

float state_distance2( float *s1, float *s2 );
float normalized_state_distance2( float *s1, float *s2 );
int state_to_index( int dimension, float state );
float index_to_state( int dimension, int index );
int indices_to_state( int *indices, float *state );
int state_to_indices( float *state, int *indices );
float index_to_action( int dimension, int index, int *n_actions );
int indices_to_action( int *indices, float *action, int *n_actions );
int value_function_index( int *indices );
int copy_state_indices( int *indices, int *result );
int sum_state_indices( int *indices1, int *indices2, int *result );
int write_array_to_file( char *name, float *array, int size );
int read_array_from_file( char *name, float *array, int size );
int init_dp();
int in_lqr_box( float *state );
float lqr_value( float *state );
int lqr_action( float *state, float *action );
float lookup_value( float *state, float *value_array );
int lqr_covers_it( int *indices, float *state, float *v_best, 
		   float *best_action, float *best_next_state );
int gradient_descent( int *indices, float *state, 
		      float *v_best, float *best_action,
		      float *best_next_state, float *value_array );
int update_action( int *indices, float *v_best, float *action,
		   float *next_state, float *value_array );
int random_search( int *indices, float *v_best, float *best_action,
		   float *best_next_state, float *value_array );
int grid_search( int *indices, float *v_best, float *best_action,
		 float *best_next_state, float *value_array, int *n_actions );
int lookup_action( float *state,
		   float a_array[N_TOTAL_STATES][N_ACTION_DIMENSIONS],
		   float *v_array,
		   float *action );


