/************************************************************************/
/*
dynamics for one link pendulum
*/
/************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "dynamics.h"

#define LENGTH (1.0f)   // length of pendulum
#define WIDTH (0.1f)    // width of pendulum
#define MASS (1.0f)     // mass of link
#define GRAVITY 9.81
#define VISCOUS_FRICTION 0.1 // viscous friction at joint

#define TIMESTEP 0.01 // Timestep of integrator

/* Score parameters */
#define STATE_PENALTY 0.01

/**********************************************************************/
/* Globals */

float the_state[N_STATE_DIMENSIONS] = { 0.0, 0.0 };
float the_time = 0;
float the_desired_state[N_STATE_DIMENSIONS] = { M_PI, 0.0 };
float moment_of_inertia_joint = 0;

/************************************************************************/

float sign(float i)
{
  if (i >=0 )
    return 1.0f;
  
  return -1.0f;
}

static float mysquare(float i)
{
  return i*i;
}

/**********************************************************************/

int init_dynamics()
{
  int i;

  moment_of_inertia_joint = MASS*(LENGTH*LENGTH + WIDTH*WIDTH)/12
    + MASS*LENGTH*LENGTH/4;
  return 1;
}

/**********************************************************************/

int set_state( float time, float *state )
{
  int i; 

  the_time = time;
  for ( i = 0; i < N_STATE_DIMENSIONS; i++ )
    the_state[i] = state[i];
  return 1;
}

/**********************************************************************/

int integrate_one_step( float *action, float *next_state )
{
  float angle, angular_velocity, total_torque;
  float angular_acceleration, new_angular_velocity;

  const double Cg = 0.72004226501331;
  const double Cc = 0.12329504549906;
  const double Cv = 0.01339875306335;
  const double Ci = 0.01944485646583;

  // Apply dynamics 
  angle = the_state[0];
  angular_velocity = the_state[1];
  


  //total_torque = appliedTorque - estParams.Cg*sin(angle) - sign(angular_velocity)*estParams.Cc - estParams.Cv*angular_velocity;
  //angular_acceleration = total_torque/estParams.I;
  //new_angular_velocity = angular_velocity + angular_acceleration*TIMESTEP;
  //angle = angle + (new_angular_velocity + angular_velocity)*TIMESTEP/2;
  //angular_velocity = new_angular_velocity;
  //time = time + TIMESTEP;


  total_torque = action[0] 
    - Cg * sinf(angle)  
    - Cc * sign(angular_velocity)
    - Cv * angular_velocity;

  angular_acceleration = total_torque/Ci;
  new_angular_velocity = angular_velocity 
    + angular_acceleration*TIMESTEP;
  
  angle += (new_angular_velocity + angular_velocity)*TIMESTEP/2;
  angular_velocity = new_angular_velocity;


  the_time += TIMESTEP;
  the_state[1] = new_angular_velocity;
  the_state[0] = angle;
  if ( next_state != NULL )
    {
      next_state[1] = the_state[1];
      next_state[0] = the_state[0];
    }
  // printf( "integrate2: %lg %lg\n", next_state[0], next_state[1] );
  return 1;
}

/*
int integrate_one_step( float *action, float *next_state )
{
  float angle, angular_velocity, total_torque;
  float angular_acceleration, new_angular_velocity;

  
  angle = the_state[0];
  angular_velocity = the_state[1];
  total_torque = action[0] - VISCOUS_FRICTION*angular_velocity
    - MASS*GRAVITY*LENGTH*0.5*sinf(angle); 
  angular_acceleration = total_torque/moment_of_inertia_joint;
  new_angular_velocity = angular_velocity 
    + angular_acceleration*TIMESTEP;
  angle += (new_angular_velocity + angular_velocity)*TIMESTEP/2;
  the_time += TIMESTEP;
  the_state[1] = new_angular_velocity;
  the_state[0] = angle;
  if ( next_state != NULL )
    {
      next_state[1] = the_state[1];
      next_state[0] = the_state[0];
    }
  // printf( "integrate2: %lg %lg\n", next_state[0], next_state[1] );
  return 1;
}
*/
/**********************************************************************/


float one_step_cost( float *state, float *action )
{
  return mysquare( action[0] )*TIMESTEP 
    + STATE_PENALTY*TIMESTEP*mysquare( state[0] - the_desired_state[0] );
}


/*float one_step_cost( float *state, float *action )
{
  return mysquare( action[0] )*TIMESTEP 
    + STATE_PENALTY*TIMESTEP*mysquare( state[0] - the_desired_state[0] );
}*/

/**********************************************************************/

