#include "kalman.h"

MAT *X;
MAT *P;

double sign2(double x) {
  if(x > 0.0)
    return 1.0;
  
  return 0.0;
}

void kalmanInit(void) {
  X = m_get(2,1);
  X->me[0][0] = 1.0;

  P = m_get(2,2);
  P->me[0][0] = 1.0;
  P->me[1][1] = 1.0;

  //printf("kalman filter intialized\n");
}

void kalmanDestroy(void) {
  m_free(X);
  m_free(P);
}

double kalmanUpdate(double dt, double angle, double prevAng, double torque) {
  int i;
  double retVel;

  //system constants
  //double Cg = 0.72004226501331;
  double Cg = 0.5;
  //double Cc = 0.12329504549906;
  double Cc = 0.2;
  double Cv = 0.01339875306335;
  //double Cv = 0.0;
  double I = 0.01944485646583;
  
  MAT *H, *Q, *R, *OF, *K, *z;
  MAT *Id, *temp, *temp2, *temp3, *temp4;
  
  //printf("in kalman update\n");

  //X = m_get(2,1);
  //P = m_get(2,2);

  Id = m_get(2,2);
  Id->me[0][0] = 1.0;
  Id->me[1][1] = 1.0;

  //printf("i allocated\n");
  
  temp = m_get(2,2);
  temp2 = m_get(2,2);
  temp3 = m_get(2,2);
  temp4 = m_get(2,2);

  H = m_get(2,2);
  H->me[0][0] = 1.0;
  H->me[1][1] = 1.0;

  Q = m_get(2,2);
  Q->me[0][0] = 1.0;
  Q->me[1][1] = 1.0;

  R = m_get(2,2);
  R->me[0][0] = 0.1;
  R->me[1][1] = 0.1;

  OF = m_get(2,2);
  K = m_get(2,2);

  z = m_get(2,1);

  z->me[0][0] = angle;
  z->me[1][0] = (angle-prevAng)/dt;

  OF->me[0][0] = 1.0;
  OF->me[0][1] = dt;
  OF->me[1][0] = -Cg*cos(angle)*dt/I;
  OF->me[1][1] = 1-Cv*dt/I;
  
  X->me[0][0] = X->me[0][0] + dt*X->me[1][0];
  X->me[1][0] = X->me[1][0] + dt*(torque-Cg*sin(X->me[0][0])-Cc*sign2(X->me[1][0])-Cv*X->me[1][0])/I;

  m_transp(OF,temp);
  m_mlt(P,temp,temp3);
  m_mlt(OF,temp3,temp4);
  m_add(temp4,Q,P);
  
  m_mlt(H,P,temp);
  m_transp(H,temp2);
  m_mlt(temp,temp2,temp3);
  m_add(temp3,R,temp3);
  m_inverse(temp3,temp3);
  m_mlt(P,temp2,temp4);
  m_mlt(temp4,temp3,K);
  
  m_mlt(H,X,temp);
  m_sub(z,temp,temp2);
  m_mlt(K,temp2,temp3);
  m_add(X,temp3,X);
  
  m_mlt(K,H,temp);
  m_sub(Id,temp,temp2);
  m_mlt(temp2,P,temp3);
  P = m_copy(temp3,P);
  
  //printf("X %lf %lf %lf\n", dt, X->me[0][0], X->me[1][0]);
  
  m_free(H); 
  m_free(R); 
  m_free(Q); 
  m_free(z);
  m_free(OF); m_free(K); m_free(Id); m_free(temp);
  m_free(temp2); m_free(temp3); m_free(temp4);

  retVel = X->me[1][0];

  return retVel;
}
