/************************************************************************/
/*
Precompute model of the effect of actions for the one link system.
At each state, save a linear model of the effect of actions on the next 
state.
next_state = next_state_0 + M*action;
model_next_states stores the vector next_state_0 and the matrix M.
*/
/************************************************************************/

#include <stdio.h>
#include <math.h>
#include "useful.h"
#include "dynamics.h"
#include "dp.h"

/************************************************************************/
/* Globals */

float model_next_states[N_TOTAL_STATES][N_STATE_DIMENSIONS][N_ACTION_DIMENSIONS+1];

/************************************************************************/
/************************************************************************/
/* initialization */

int init()
{
  init_dynamics();

  return 0;
}

/************************************************************************/
/************************************************************************/
/************************************************************************/
/* main */

int main(int argc, char **argv)
{
  int i, j;
  int index;
  int indices[N_STATE_DIMENSIONS];
  float state[N_STATE_DIMENSIONS];
  float next_state_0[N_STATE_DIMENSIONS];
  float next_state_1[N_STATE_DIMENSIONS];
  float action[N_ACTION_DIMENSIONS];
  float delta = 1.0;

  init();
	
  for( indices[0] = 0; indices[0] < n_states[0]; (indices[0])++ )
    for( indices[1] = 0; indices[1] < n_states[1]; (indices[1])++ )
      {
	indices_to_state( indices, state );
	index = value_function_index( indices );

	for( i = 0; i < N_ACTION_DIMENSIONS; i++ )
	  action[i] = 0;
	set_state( 0.0, state );
	integrate_one_step( action, next_state_0 );
	for( j = 0; j < N_STATE_DIMENSIONS; j++ )
	  model_next_states[index][j][0] = next_state_0[j];

	for( i = 0; i < N_ACTION_DIMENSIONS; i++ )
	  {
	    action[i] = delta;
	    set_state( 0.0, state );
	    integrate_one_step( action, next_state_1 );
	    for( j = 0; j < N_STATE_DIMENSIONS; j++ )
	      model_next_states[index][j][i+1]
		= (next_state_1[j] - next_state_0[j])/delta;
	    action[i] = 0;
	  }
      }
  
  write_array_to_file( "model_next_states", &(model_next_states[0][0][0]),
		       N_TOTAL_STATES*(N_ACTION_DIMENSIONS+1)
		       *N_STATE_DIMENSIONS );

  return 0;
}

/************************************************************************/

