/***********************************************************************
Read in 1D value function and policy, Write out as ascii matrices
for Matlab.

***********************************************************************/

#include <stdio.h>
#include <math.h>
#include "dynamics.h"
#include "dp.h"

/**********************************************************************/
/************************************************************************/
/* Globals */

// I am using 1D arrays for speed and simplicity
/* value functions: I double buffer these using v_k and v_next */
float v1[N_TOTAL_STATES];
float v2[N_TOTAL_STATES];
/* the current policy */
float u_array[N_TOTAL_STATES][N_ACTION_DIMENSIONS];
/* Where the policy takes us */
float next_state_array[N_TOTAL_STATES][N_STATE_DIMENSIONS];

/* pointers for swapping value functions */
float *v_k, *v_next;

/**********************************************************************/
/**********************************************************************/
/************************************************************************/

int read_array_from_file( char *name, float *array, int size )
{
  int ip, iv, i1d;
  FILE *input;

  input = fopen( name, "r" );
  if ( input == NULL )
    {
      fprintf( stderr, "Couldn't open file %s.\n", name );
      exit( -1 );
    }

  fread( (void *) array, sizeof ( float ), size, input );
  fclose( input );
  return 0;
}

/************************************************************************/
/* figure out how to index 1D value array */

int value_function_index_old( int ip, int iv )
{
  return ip*N_VELOCITIES_0 + iv;
}

/************************************************************************/
/* print array to output file */

int print_array_to_file( char *name, float *array )
{
  int ip, iv, i1d;
  FILE *output;

  output = fopen( name, "w" );
  if ( output == NULL )
    {
      fprintf( stderr, "Couldn't open file %s.\n", name );
      exit( -1 );
    }

  for( ip = 0; ip < N_POSITIONS_0; ip++ )
    {
      for( iv = 0; iv < N_VELOCITIES_0; iv++ )
	{
	  i1d = value_function_index_old( ip, iv );
	  fprintf( output, "%g ", array[i1d]);
	}
      fprintf(output, "\n");
    }
  fclose( output );
  return 0;
}

/************************************************************************/
/* initialization */

int init()
{
  int i, j;
	
  v_k = &(v1[0]);
  v_next = &(v2[0]);
		
  for( i = 0; i < N_TOTAL_STATES; i++ )
    {
      v_next[i] = 0;
      v_k[i] = 0;
      for( j = 0; j < N_ACTION_DIMENSIONS; j++ )
	u_array[i][j] = 0;
      for( j = 0; j < N_STATE_DIMENSIONS; j++ )
	next_state_array[i][j] = 0;
    }

  return 0;
}

/**********************************************************************/

main()
{

  init();

  read_array_from_file( "v_k", v_k, N_TOTAL_STATES );
  read_array_from_file( "u_array", &(u_array[0][0]),
			N_TOTAL_STATES*N_ACTION_DIMENSIONS );

  print_array_to_file( "v_k2", v_k );
  print_array_to_file( "u_array2", &(u_array[0][0]) );
}

/**********************************************************************/
