/***********************************************************************
Mimic ODE simulation for the one link pendulum
Use value function to determine action.

***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "useful.h"
#include "dynamics.h"
#include "dp.h"

/**********************************************************************/
/************************************************************************/
/* Globals */

int n_actions[N_ACTION_DIMENSIONS] = { 1001 };

// I am using 1D arrays for speed and simplicity
float v1[N_TOTAL_STATES];

/* pointers for swapping value functions */
float *v_k;

float *model_next_states = NULL; /* Need this to make compiler happy. */

/**********************************************************************/
/**********************************************************************/
/************************************************************************/
/************************************************************************/
/* initialization */

int init()
{
  v_k = &(v1[0]);
		
  read_array_from_file( "v_k", v_k, N_TOTAL_STATES );

  init_dynamics();
  init_dp();


  return 0;
}

/************************************************************************/
/************************************************************************/
/* sweep function */

float search_for_action_using_value_function( float *state, 
					      float *best_action ) 
{
  float next_state[N_STATE_DIMENSIONS];
  float action[N_ACTION_DIMENSIONS];
  int action_indices[N_ACTION_DIMENSIONS];
  float v_best;
  float value;
  int i;

  v_best = INFINITE_COST;

  for( action_indices[0] = 0; action_indices[0] < n_actions[0];
       (action_indices[0])++ )
    {
      indices_to_action( action_indices, action, n_actions );
      
      /* Apply dynamics */
      set_state( 0.0, state );
      integrate_one_step( action, next_state );

      /* Get cost */
      if ( !lqr_covers_it( NULL, next_state, &value, NULL, NULL ) )
	value = lookup_value( next_state, v_k );
      value += one_step_cost( state, action );

      /* Improve it */
      gradient_descent( NULL, state, &value, action, next_state, v_k );
      // gradient_descent( NULL, state, &value, action, next_state, v_k );

      if( value < v_best )
	{
	  v_best = value;
	  for( i = 0; i < N_ACTION_DIMENSIONS; i++ )
	    best_action[i] = action[i];
	}
    }

  if ( v_best < COST_CUTOFF )
    {
      return v_best;
    }

  for( i = 0; i < N_ACTION_DIMENSIONS; i++ )
    best_action[i] = BAD_U;
  return v_best;
}

/**********************************************************************/

main()
{
  int i;
  float score = 0;
  FILE *stream;
  float action[N_ACTION_DIMENSIONS];
  float my_time = 0;
  float my_state[N_STATE_DIMENSIONS] = { 0, 0 };

  init();

  stream = fopen( "traj", "w" );
  if ( stream == NULL )
    {
      fprintf( stderr, "Can't open data file.\n" );
      exit( -1 );
    }

  /* Initial state already set in dynamics.c */

  for( i = 0; ; i++ )
    {
      search_for_action_using_value_function( my_state, action );
      fprintf( stream, "%g %g %g %g %g\n", my_time, my_state[0], 
	       my_state[1], action[0], score );
      printf( "%3d %6.2f %6.2f %6.2f %6.2f %6.2f\n", i,
	      my_time, my_state[0], 
	      my_state[1], action[0], score );
      score += one_step_cost( my_state, action );
      set_state( my_time, my_state );
      integrate_one_step( action, my_state );
      my_time = the_time;

      if ( the_time > 9.99999 )
	{
	  fclose( stream );
	  exit( 0 );
	}
    }
}

/**********************************************************************/
